import logging

from django.shortcuts import render
from rest_framework.views import APIView

from vnpay.utils import VnPay

logger = logging.getLogger(__name__)


class VnPayPaymentReturnApi(APIView):

    def get(self, request, *args, **kwargs):
        input_data = request.GET
        if not input_data:
            return render(request, 'vnpay/payment_return.html', {'title': 'Kết quả thanh toán', 'result': ''})

        vnp = VnPay()
        response_data = vnp.validate_response_data(input_data)

        order_id = response_data['vnp_TxnRef']
        amount = int(response_data['vnp_Amount']) / 100
        order_desc = response_data['vnp_OrderInfo']
        vnp_transaction_no = response_data['vnp_TransactionNo']
        vnp_response_code = response_data['vnp_ResponseCode']

        context = {
            'title': 'Kết quả thanh toán',
            'result': 'Thành công',
            'order_id': order_id,
            'amount': amount,
            'order_desc': order_desc,
            'vnp_TransactionNo': vnp_transaction_no,
            'vnp_ResponseCode': vnp_response_code,
        }

        if vnp.validate_hash(response_data):
            if not vnp_response_code == '00':
                context['result'] = 'Không thành công'
        else:
            context['result'] = 'Không thành công'
            context['msg'] = 'Sai checksum'

        return render(request, 'vnpay/payment_return.html', context)



import json
import logging

from rest_framework import serializers
from rest_framework.response import Response
from rest_framework.views import APIView

from vnpay.models import Billing
from vnpay.utils import VnPay

logger = logging.getLogger(__name__)


def logger_json(order_id, data):
    header = f'VNPAY_IPN {order_id}' if order_id else 'VNPAY_IPN'
    logger.info(f'{header} {json.dumps(data)}')


class VnPayPaymentIpnResponseSerializer(serializers.Serializer):
    RspCode = serializers.CharField()
    Message = serializers.CharField()


class VnPayPaymentIpnApi(APIView):

    def get_response_data(self, input_data):
        vnp = VnPay()
        response_data = vnp.validate_response_data(input_data)

        order_id = response_data['vnp_TxnRef']
        amount = int(response_data['vnp_Amount']) / 100
        vnp_response_code = response_data['vnp_ResponseCode']
        transaction_id = response_data['vnp_TransactionNo']

        if vnp.validate_hash(response_data):
            try:
                billing = Billing.objects.get(reference_number=order_id)
                if billing.amount != amount:
                    response_data = {'RspCode': '04', 'Message': 'Invalid Amount'}
                    return order_id, response_data
                if billing.status == 'NEW':
                    # TODO refactor and make reuse
                    succeeded = vnp_response_code == '00'
                    billing.finalize(succeeded=succeeded)
                    billing.status = 'CONFIRMED'
                    billing.result_payment = 'VNPAY_' + vnp_response_code
                    billing.transaction_id = transaction_id
                    billing.save(update_fields=['status', 'result_payment', 'transaction_id'])

                    # Return VNPAY: Merchant update success
                    response_data = {'RspCode': '00', 'Message': 'Confirm Success'}
                    return order_id, response_data
                else:
                    # Already Update
                    response_data = {'RspCode': '02', 'Message': 'Order Already Update'}
                    return order_id, response_data
            except Billing.DoesNotExist:
                response_data = {'RspCode': '01', 'Message': 'Order not found'}
                return order_id, response_data

        # Invalid Signature
        response_data = {'RspCode': '97', 'Message': 'Invalid Signature'}
        return order_id, response_data

    def get(self, request, *args, **kwargs):
        order_id = None

        try:
            input_data = request.GET
            if input_data:
                order_id, response_data = self.get_response_data(input_data)
            else:
                response_data = {'RspCode': '99', 'Message': 'Invalid request'}
        except Exception:  # pragma: no cover
            response_data = {'RspCode': '99', 'Message': 'Unknown error'}

        serializers = VnPayPaymentIpnResponseSerializer(data=response_data)
        serializers.is_valid(raise_exception=True)

        logger_json(order_id, response_data)
        return Response(serializers.data)



from django.urls import path

from vnpay.api.payment_ipn import VnPayPaymentIpnApi
from vnpay.api.payment_return import VnPayPaymentReturnApi
from vnpay.api.payment_url import VnPayPaymentUrlApi

urlpatterns = [
    path('payment_url/', VnPayPaymentUrlApi.as_view(), name='payment-url'),
    path('payment_ipn/', VnPayPaymentIpnApi.as_view(), name='payment-ipn'),
    path('payment_return/', VnPayPaymentReturnApi.as_view(), name='payment-return'),
]



from django.utils import timezone

from rest_framework import serializers
from rest_framework.generics import GenericAPIView
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from vnpay.models import Billing


class VnPayPaymentUrlSerializer(serializers.Serializer):
    amount = serializers.IntegerField()


class VnPayPaymentUrlApi(GenericAPIView):
    serializer_class = VnPayPaymentUrlSerializer
    permission_classes = (IsAuthenticated,)

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        billing = Billing.objects.create(
            status='NEW',
            currency='VND',
            pay_by=self.request.user,
            amount=serializer.validated_data['amount'],
            reference_number=timezone.now().strftime('%Y%m%d%H%M%S'),
        )

        payment_url = billing.get_payment_url(request)
        return Response({'payment_url': payment_url})



# Generated by Django 4.1.7 on 2023-04-08 09:56

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Billing',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('amount', models.DecimalField(decimal_places=2, max_digits=10)),
                ('currency', models.CharField(default='VND', max_length=3)),
                ('pay_at', models.DateTimeField(blank=True, null=True)),
                ('reference_number', models.CharField(blank=True, help_text='Reference number from payment gateway', max_length=100, null=True)),
                ('status', models.CharField(choices=[('NEW', 'NEW'), ('CONFIRMED', 'CONFIRMED')], default='NEW', max_length=20)),
                ('result_payment', models.CharField(choices=[('VNPAY_00', 'VNPAY SUCCESS'), ('VNPAY_05', 'VNPAY BAD_PASSWORD_EXCEEDING'), ('VNPAY_06', 'VNPAY WRONG_PASSWORD'), ('VNPAY_07', 'VNPAY SUSPICIOUS_TRANSACTION'), ('VNPAY_12', 'VNPAY CARD_IS_LOCKED'), ('VNPAY_09', 'VNPAY NO_INTERNET_BANKING'), ('VNPAY_10', 'VNPAY VERIFICATION_EXCEEDING'), ('VNPAY_11', 'VNPAY TIMEOUT_PAYMENT'), ('VNPAY_24', 'VNPAY CUSTOMER_CANCEL'), ('VNPAY_51', 'VNPAY NOT_ENOUGH_BALANCE'), ('VNPAY_65', 'VNPAY TRANSACTION_PER_DAY_EXCEEDING'), ('VNPAY_75', 'VNPAY BANK_MAINTENANCE'), ('VNPAY_99', 'VNPAY OTHER_ERRORS')], max_length=50, null=True)),
                ('is_paid', models.BooleanField(default=False)),
                ('transaction_id', models.CharField(blank=True, max_length=255, null=True)),
                ('pay_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



import pytz

from datetime import datetime
from django.contrib import admin

from vnpay.models import Billing


def get_vn_datetime(input_datetime):
    if not input_datetime:
        return '-'
    vn_timezone = pytz.timezone('Asia/Ho_Chi_Minh')
    utc_offset = datetime.now(vn_timezone).utcoffset()
    return (input_datetime + utc_offset).strftime('%B %d, %Y, %I:%M %p')


class BillingAdmin(admin.ModelAdmin):
    list_display = ('id', 'pay_by', 'reference_number', 'amount', 'status', 'pay_at_vn', 'created_at_vn')
    search_fields = ('id', 'pay_by__username')
    readonly_fields = ('pay_at_vn', 'created_at_vn')

    def pay_at_vn(self, obj):
        return get_vn_datetime(obj.pay_at)

    def created_at_vn(self, obj):
        return get_vn_datetime(obj.created_at)


# Register your models here.
admin.site.register(Billing, BillingAdmin)
